@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2006 the xine project
+ * Copyright (C) 2000-2013 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -92,6 +92,11 @@ extern "C" {
 #define MM_ACCEL_X86_MMXEXT     0x20000000
 #define MM_ACCEL_X86_SSE        0x10000000
 #define MM_ACCEL_X86_SSE2       0x08000000
+#define MM_ACCEL_X86_SSE3       0x04000000
+#define MM_ACCEL_X86_SSSE3      0x02000000
+#define MM_ACCEL_X86_SSE4       0x01000000
+#define MM_ACCEL_X86_SSE42      0x00800000
+#define MM_ACCEL_X86_AVX        0x00400000
 
 /* powerpc accelerations and features */
 #define MM_ACCEL_PPC_ALTIVEC    0x04000000
@@ -257,6 +262,9 @@ extern void (*yuy2_to_yv12)
 #define SCALEFACTOR 65536
 #define CENTERSAMPLE 128
 
+/* These conversions are normalised for the MPEG Y'CbCr colourspace.
+ * (Yes, we know that we call it YUV elsewhere.)
+ */
 #define COMPUTE_Y(r, g, b) \
   (unsigned char) \
   ((y_r_table[r] + y_g_table[g] + y_b_table[b]) / SCALEFACTOR)
@@ -299,6 +307,15 @@ extern int v_r_table[256] XINE_PROTECTED;
 extern int v_g_table[256] XINE_PROTECTED;
 extern int v_b_table[256] XINE_PROTECTED;
 
+/* TJ. direct sliced rgb -> yuy2 conversion */
+extern void *rgb2yuy2_alloc (int color_matrix, const char *format) XINE_PROTECTED;
+extern void  rgb2yuy2_free (void *rgb2yuy2) XINE_PROTECTED;
+extern void  rgb2yuy2_slice (void *rgb2yuy2, const uint8_t *in, int ipitch, uint8_t *out, int opitch,
+  int width, int height) XINE_PROTECTED;
+extern void  rgb2yuy2_palette (void *rgb2yuy2, const uint8_t *pal, int num_colors, int bits_per_pixel)
+  XINE_PROTECTED;
+
+
 /* frame copying functions */
 extern void yv12_to_yv12
   (const unsigned char *y_src, int y_src_pitch, unsigned char *y_dst, int y_dst_pitch,
