@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2013 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -146,4 +146,8 @@
 #  define AVCODEC_MAX_AUDIO_FRAME_SIZE 192000
 #endif
 
+#if LIBAVCODEC_VERSION_INT < ((52<<16)|(66<<8))
+#  define avcodec_get_edge_width() (16)
+#endif
+
 #endif /* XINE_AVCODEC_COMPAT_H */
