@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2006 the xine project
+ * Copyright (C) 2000-2012 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -31,15 +31,21 @@
 #include <unistd.h>
 #include <string.h>
 
+#ifdef HAVE_FFMPEG_AVUTIL_H
+#  include <mem.h>
+#else
+#  include <libavutil/mem.h>
+#endif
+
 #define LOG_MODULE "demux_mpeg_block"
 #define LOG_VERBOSE
 /*
 #define LOG
 */
 
-#include "xine_internal.h"
-#include "xineutils.h"
-#include "demux.h"
+#include <xine/xine_internal.h>
+#include <xine/xineutils.h>
+#include <xine/demux.h>
 
 #define NUM_PREVIEW_BUFFERS   250
 #define DISC_TRESHOLD       90000
@@ -1449,37 +1455,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   }
   break;
 
-  case METHOD_BY_EXTENSION: {
-    char *ending;
-
-    const char *const mrl = input->get_mrl (input);
-
-    if(!strncmp(mrl, "vcd:", 4)) {
-      this->blocksize = 2324;
-      demux_mpeg_block_accept_input (this, input);
-    } else if(!strncmp(mrl, "dvd:", 4) || !strncmp(mrl, "pvr:", 4)) {
-      this->blocksize = 2048;
-      demux_mpeg_block_accept_input (this, input);
-    } else {
-      ending = strrchr(mrl, '.');
-
-      if (!ending) {
-        free (this);
-        return NULL;
-      }
-      if ( (!strncasecmp (ending, ".vob", 4)) ||
-	   (!strncmp((ending + 3), "mpeg2", 5)) ||
-	   (!strncmp((ending + 3), "mpeg1", 5)) ) {
-        this->blocksize = 2048;
-        demux_mpeg_block_accept_input(this, input);
-      } else {
-        free (this);
-        return NULL;
-      }
-    }
-  }
-  break;
-
+  case METHOD_BY_MRL:
   case METHOD_EXPLICIT: {
 
     this->blocksize = input->get_blocksize(input);
@@ -1506,29 +1482,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static const char *get_description (demux_class_t *this_gen) {
-  return "DVD/VOB demux plugin";
-}
-
-static const char *get_identifier (demux_class_t *this_gen) {
-  return "MPEG_BLOCK";
-}
-
-static const char *get_extensions (demux_class_t *this_gen) {
-  return "vob";
-}
-
-static const char *get_mimetypes (demux_class_t *this_gen) {
-  return NULL;
-}
-
-static void class_dispose (demux_class_t *this_gen) {
-
-  demux_mpeg_block_class_t *this = (demux_mpeg_block_class_t *) this_gen;
-
-  free (this);
- }
-
 static void *init_plugin (xine_t *xine, void *data) {
 
   demux_mpeg_block_class_t     *this;
@@ -1538,11 +1491,11 @@ static void *init_plugin (xine_t *xine, void *data) {
   this->xine   = xine;
 
   this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.get_description = get_description;
-  this->demux_class.get_identifier  = get_identifier;
-  this->demux_class.get_mimetypes   = get_mimetypes;
-  this->demux_class.get_extensions  = get_extensions;
-  this->demux_class.dispose         = class_dispose;
+  this->demux_class.description     = N_("DVD/VOB demux plugin");
+  this->demux_class.identifier      = "MPEG_BLOCK";
+  this->demux_class.mimetypes       = NULL;
+  this->demux_class.extensions      = "vob vcd:/ dvd:/ pvr:/";
+  this->demux_class.dispose         = default_demux_class_dispose;
 
   return this;
 }
@@ -1556,6 +1509,6 @@ static const demuxer_info_t demux_info_mpeg_block = {
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 26, "mpeg_block", XINE_VERSION_CODE, &demux_info_mpeg_block, init_plugin },
+  { PLUGIN_DEMUX, 27, "mpeg_block", XINE_VERSION_CODE, &demux_info_mpeg_block, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
