@@ -1,6 +1,6 @@
 
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2013 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -181,7 +181,10 @@
 #define BODY_SIZE (188 - 4)
 /* more PIDS are needed due "auto-detection". 40 spare media entries  */
 #define MAX_PIDS ((BODY_SIZE - 1 - 13) / 4) + 40
-#define MAX_PMTS ((BODY_SIZE - 1 - 13) / 4) + 10
+
+#define MAX_PMTS 128
+#define PAT_BUF_SIZE (4 * MAX_PMTS + 20)
+
 #define SYNC_BYTE   0x47
 
 #define MIN_SYNCS 3
@@ -367,13 +370,15 @@ typedef struct {
   demux_ts_media   media[MAX_PIDS];
 
   /* PAT */
+  uint8_t          pat[PAT_BUF_SIZE];
+  int              pat_write_pos;
   uint32_t         last_pat_crc;
   uint32_t         transport_stream_id;
   /* programs */
   uint32_t         program_number[MAX_PMTS];
   uint32_t         pmt_pid[MAX_PMTS];
   uint8_t         *pmt[MAX_PMTS];
-  uint8_t         *pmt_write_ptr[MAX_PMTS];
+  int              pmt_write_pos[MAX_PMTS];
   uint32_t         last_pmt_crc;
   /*
    * Stuff to do with the transport header. As well as the video
@@ -455,7 +460,7 @@ static int demux_ts_dynamic_pmt_find (demux_ts_t *this,
   if (i < MAX_PIDS) {
     /* prepare new media descriptor */
 #ifdef LOG_DYNAMIC_PMT
-    char *name = "";
+    const char *name = "";
     if (type == BUF_VIDEO_BASE) name = "video";
     else if (type == BUF_AUDIO_BASE) name = "audio";
     else if (type == BUF_SPU_BASE) name = "subtitle";
@@ -535,7 +540,7 @@ static void demux_ts_dynamic_pmt_clean (demux_ts_t *this) {
     } else {
       /* drop this no longer needed media descriptor */
 #ifdef LOG_DYNAMIC_PMT
-      char *name = "";
+      const char *name = "";
       if (type == BUF_VIDEO_BASE) name = "video";
       else if (type == BUF_AUDIO_BASE) name = "audio";
       else if (type == BUF_SPU_BASE) name = "subtitle";
@@ -785,6 +790,27 @@ static void demux_ts_flush_media(demux_ts_media *m)
   demux_ts_send_buffer(m, BUF_FLAG_FRAME_END);
 }
 
+static void post_sequence_end(fifo_buffer_t *fifo, uint32_t video_type) {
+
+  if (video_type == BUF_VIDEO_H264 ||
+      video_type == BUF_VIDEO_MPEG ||
+      video_type == BUF_VIDEO_VC1) {
+
+    buf_element_t *buf = fifo->buffer_pool_try_alloc(fifo);
+    if (buf) {
+      buf->type = video_type;
+      buf->size = 4;
+      buf->decoder_flags = BUF_FLAG_FRAME_END;
+      buf->content[0] = 0x00;
+      buf->content[1] = 0x00;
+      buf->content[2] = 0x01;
+      buf->content[3] = (video_type == BUF_VIDEO_MPEG) ? 0xb7 : 0x0a;
+      fifo->put(fifo, buf);
+    }
+  }
+}
+
+
 static void demux_ts_flush(demux_ts_t *this)
 {
   unsigned int i;
@@ -792,20 +818,22 @@ static void demux_ts_flush(demux_ts_t *this)
     demux_ts_flush_media(&this->media[i]);
     this->media[i].corrupted_pes = 1;
   }
+
+  /* append sequence end code to video stream */
+  if (this->videoPid != INVALID_PID)
+    post_sequence_end(this->stream->video_fifo, this->media[this->videoMedia].type);
 }
 
 /*
  * demux_ts_parse_pat
  *
  * Parse a program association table (PAT).
- * The PAT is expected to be exactly one section long,
- * and that section is expected to be contained in a single TS packet.
+ * The PAT is expected to be exactly one section long.
  *
- * The PAT is assumed to contain a single program definition, though
- * we can cope with the stupidity of SPTSs which contain NITs.
+ * We can cope with the stupidity of SPTSs which contain NITs.
  */
 static void demux_ts_parse_pat (demux_ts_t*this, unsigned char *original_pkt,
-                                unsigned char *pkt, unsigned int pusi) {
+                                unsigned char *pkt, unsigned int pusi, int len) {
 #ifdef TS_PAT_LOG
   uint32_t       table_id;
   uint32_t       version_number;
@@ -824,47 +852,67 @@ static void demux_ts_parse_pat (demux_ts_t*this, unsigned char *original_pkt,
   unsigned int   pmt_pid;
   unsigned int   program_count;
 
-  /*
-   * A PAT in a single section should start with a payload unit start
-   * indicator set.
-   */
-  if (!pusi) {
+  /* reassemble the section */
+  if (pusi) {
+    this->pat_write_pos = 0;
+    /* offset the section by n + 1 bytes. this is sometimes used to let it end
+       at an exact TS packet boundary */
+    len -= (unsigned int)pkt[0] + 1;
+    pkt += (unsigned int)pkt[0] + 1;
+    if (len < 1) {
+      xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
+        "demux_ts: demux error! PAT with invalid pointer\n");
+      return;
+    }
+  } else {
+    if (!this->pat_write_pos)
+      return;
+  }
+  if (len > PAT_BUF_SIZE - this->pat_write_pos)
+    len = PAT_BUF_SIZE - this->pat_write_pos;
+  memcpy (this->pat + this->pat_write_pos, pkt, len);
+  this->pat_write_pos +=len;
+
+  /* lets see if we got the section length already */
+  pkt = this->pat;
+  if (this->pat_write_pos < 3)
+    return;
+  section_length = ((((uint32_t)pkt[1] << 8) | pkt[2]) & 0x03ff) + 3;
+  /* this should be at least the head plus crc */
+  if (section_length < 8 + 4) {
+    this->pat_write_pos = 0;
+    return;
+  }
+  /* and it should fit into buf */
+  if (section_length > PAT_BUF_SIZE) {
     xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
-	     "demux_ts: demux error! PAT without payload unit start indicator\n");
+      "demux_ts: PAT too large (%d bytes)\n", section_length);
+    this->pat_write_pos = 0;
     return;
   }
 
-  /*
-   * sections start with a pointer. Skip it!
-   */
-  pkt += pkt[4];
-  if (pkt - original_pkt > PKT_SIZE) {
-    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
-	     "demux_ts: demux error! PAT with invalid pointer\n");
+  /* lets see if we got the section complete */
+  if (this->pat_write_pos < section_length)
     return;
-  }
+
+  /* OK now parse it */
+  this->pat_write_pos = 0;
 #ifdef TS_PAT_LOG
-  table_id = (unsigned int)pkt[5] ;
+  table_id                  = (unsigned int)pkt[0];
 #endif
-  section_syntax_indicator = (((unsigned int)pkt[6] >> 7) & 1) ;
-  section_length = (((unsigned int)pkt[6] & 0x03) << 8) | pkt[7];
-  transport_stream_id = ((uint32_t)pkt[8] << 8) | pkt[9];
+  section_syntax_indicator  = (pkt[1] >> 7) & 0x01;
+  transport_stream_id       = ((uint32_t)pkt[3] << 8) | pkt[4];
 #ifdef TS_PAT_LOG
-  version_number = ((uint32_t)pkt[10] >> 1) & 0x1f;
+  version_number            = ((uint32_t)pkt[5] >> 1) & 0x1f;
 #endif
-  current_next_indicator = ((uint32_t)pkt[10] & 0x01);
-  section_number = (uint32_t)pkt[11];
-  last_section_number = (uint32_t)pkt[12];
-  crc32 = (uint32_t)pkt[4+section_length] << 24;
-  crc32 |= (uint32_t)pkt[5+section_length] << 16;
-  crc32 |= (uint32_t)pkt[6+section_length] << 8;
-  crc32 |= (uint32_t)pkt[7+section_length] ;
+  current_next_indicator    =  pkt[5] & 0x01;
+  section_number            =  pkt[6];
+  last_section_number       =  pkt[7];
 
 #ifdef TS_PAT_LOG
   printf ("demux_ts: PAT table_id: %.2x\n", table_id);
   printf ("              section_syntax: %d\n", section_syntax_indicator);
-  printf ("              section_length: %d (%#.3x)\n",
-          section_length, section_length);
+  printf ("              section_length: %d (%#.3x)\n", section_length, section_length);
   printf ("              transport_stream_id: %#.4x\n", transport_stream_id);
   printf ("              version_number: %d\n", version_number);
   printf ("              c/n indicator: %d\n", current_next_indicator);
@@ -872,24 +920,22 @@ static void demux_ts_parse_pat (demux_ts_t*this, unsigned char *original_pkt,
   printf ("              last_section_number: %d\n", last_section_number);
 #endif
 
-  if ((section_syntax_indicator != 1) || !(current_next_indicator)) {
+  if ((section_syntax_indicator != 1) || !current_next_indicator)
     return;
-  }
-
-  if (pkt - original_pkt > BODY_SIZE - 1 - 3 - section_length) {
-    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
-	     "demux_ts: FIXME: (unsupported )PAT spans multiple TS packets\n");
-    return;
-  }
 
   if ((section_number != 0) || (last_section_number != 0)) {
     xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
-	     "demux_ts: FIXME: (unsupported) PAT consists of multiple (%d) sections\n", last_section_number);
+      "demux_ts: FIXME (unsupported) PAT consists of multiple (%d) sections\n", last_section_number);
     return;
   }
 
   /* Check CRC. */
-  calc_crc32 = htonl(av_crc(this->class->av_crc, 0xffffffff, pkt+5, section_length+3-4));
+  crc32  = (uint32_t) pkt[section_length - 4] << 24;
+  crc32 |= (uint32_t) pkt[section_length - 3] << 16;
+  crc32 |= (uint32_t) pkt[section_length - 2] << 8;
+  crc32 |= (uint32_t) pkt[section_length - 1];
+
+  calc_crc32 = htonl (av_crc (this->class->av_crc, 0xffffffff, pkt, section_length - 4));
   if (crc32 != calc_crc32) {
     xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
 	     "demux_ts: demux error! PAT with invalid CRC32: packet_crc32: %.8x calc_crc32: %.8x\n",
@@ -920,8 +966,8 @@ static void demux_ts_parse_pat (demux_ts_t*this, unsigned char *original_pkt,
    * Process all programs in the program loop.
    */
   program_count = 0;
-  for (program = pkt + 13;
-       (program < pkt + 13 + section_length - 9) && (program_count < MAX_PMTS);
+  for (program = pkt + 8;
+       (program < pkt + section_length - 4) && (program_count < MAX_PMTS);
        program += 4) {
     program_number = ((unsigned int)program[0] << 8) | program[1];
     pmt_pid = (((unsigned int)program[2] & 0x1f) << 8) | program[3];
@@ -950,7 +996,7 @@ static void demux_ts_parse_pat (demux_ts_t*this, unsigned char *original_pkt,
       if (this->pmt[program_count] != NULL) {
 	free(this->pmt[program_count]);
 	this->pmt[program_count] = NULL;
-	this->pmt_write_ptr[program_count] = NULL;
+	this->pmt_write_pos[program_count] = 0;
       }
     }
 #ifdef TS_PAT_LOG
@@ -1311,6 +1357,35 @@ static void demux_ts_buffer_pes(demux_ts_t*this, unsigned char *ts,
       demux_ts_flush_media(m);
       /* skip rest data - there shouldn't be any */
       m->corrupted_pes = 1;
+    } else {
+
+      /* If video data ends to sequence end code, flush buffer. */
+      /* (there won't be any more data -> no pusi -> last buffer is never flushed) */
+      if (m->pid == this->videoPid && m->buf->size > 4 && m->buf->mem[m->buf->size-4] == 0) {
+
+        if (m->type == BUF_VIDEO_MPEG) {
+          if (!memcmp(&m->buf->mem[m->buf->size-4], "\x00\x00\x01\xb7", 4)) {
+            xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+                    "demux_ts: PID 0x%.4x: flushing after MPEG end of sequence code\n", m->pid);
+            demux_ts_flush_media(m);
+          }
+
+        } else if (m->type == BUF_VIDEO_H264) {
+          if ((!memcmp(&m->buf->mem[m->buf->size-4], "\x00\x00\x01\x0a", 4)) ||
+              (m->buf->size > 5 &&
+               !memcmp(&m->buf->mem[m->buf->size-5], "\x00\x00\x01\x0a", 4))) {
+            xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+                    "demux_ts: PID 0x%.4x: flushing after H.264 end of sequence code\n", m->pid);
+            demux_ts_flush_media(m);
+          }
+        } else if (m->type == BUF_VIDEO_VC1) {
+          if (!memcmp(&m->buf->mem[m->buf->size-4], "\x00\x00\x01\x0a", 4)) {
+            xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+                    "demux_ts: PID 0x%.4x: flushing after VC-1 end of sequence code\n", m->pid);
+            demux_ts_flush_media(m);
+          }
+        }
+      }
     }
   }
 }
@@ -1392,8 +1467,7 @@ static int apid_check(demux_ts_t*this, unsigned int pid) {
 /*
  * NAME demux_ts_parse_pmt
  *
- * Parse a PMT. The PMT is expected to be exactly one section long,
- * and that section is expected to be contained in a single TS packet.
+ * Parse a PMT. The PMT is expected to be exactly one section long.
  *
  * In other words, the PMT is assumed to describe a reasonable number of
  * video, audio and other streams (with descriptors).
@@ -1403,6 +1477,7 @@ static void demux_ts_parse_pmt (demux_ts_t     *this,
                                 unsigned char *originalPkt,
                                 unsigned char *pkt,
                                 unsigned int   pusi,
+                                int            plen,
                                 uint32_t       program_count) {
 
 #ifdef TS_PMT_LOG
@@ -1410,7 +1485,7 @@ static void demux_ts_parse_pmt (demux_ts_t     *this,
   uint32_t       version_number;
 #endif
   uint32_t       section_syntax_indicator;
-  uint32_t       section_length = 0; /* to calm down gcc */
+  uint32_t       section_length;
   uint32_t       program_number;
   uint32_t       current_next_indicator;
   uint32_t       section_number;
@@ -1422,154 +1497,119 @@ static void demux_ts_parse_pmt (demux_ts_t     *this,
   unsigned int	 pid;
   unsigned char *stream;
   unsigned int	 i;
-  int		 count;
-  uint8_t	*ptr = NULL;
-  unsigned char  len;
-  unsigned int   offset=0;
   int            mi;
 
-  /*
-   * A new section should start with the payload unit start
-   * indicator set. We allocate some mem (max. allowed for a PM section)
-   * to copy the complete section into one chunk.
-   */
+  /* reassemble the section */
   if (pusi) {
-    pkt+=pkt[4]; /* pointer to start of section */
-    offset=1;
-
-    free(this->pmt[program_count]);
-    this->pmt[program_count] = (uint8_t *) calloc(4096, sizeof(unsigned char));
-    this->pmt_write_ptr[program_count] = this->pmt[program_count];
-
-#ifdef TS_PMT_LOG
-    table_id                  =  pkt[5] ;
-#endif
-    section_syntax_indicator  = (pkt[6] >> 7) & 0x01;
-    section_length            = (((uint32_t) pkt[6] << 8) | pkt[7]) & 0x03ff;
-    program_number            =  ((uint32_t) pkt[8] << 8) | pkt[9];
-#ifdef TS_PMT_LOG
-    version_number            = (pkt[10] >> 1) & 0x1f;
-#endif
-    current_next_indicator    =  pkt[10] & 0x01;
-    section_number            =  pkt[11];
-    last_section_number       =  pkt[12];
-
-#ifdef TS_PMT_LOG
-    printf ("demux_ts: PMT table_id: %2x\n", table_id);
-    printf ("              section_syntax: %d\n", section_syntax_indicator);
-    printf ("              section_length: %d (%#.3x)\n",
-            section_length, section_length);
-    printf ("              program_number: %#.4x\n", program_number);
-    printf ("              version_number: %d\n", version_number);
-    printf ("              c/n indicator: %d\n", current_next_indicator);
-    printf ("              section_number: %d\n", section_number);
-    printf ("              last_section_number: %d\n", last_section_number);
-#endif
-
-    if ((section_syntax_indicator != 1) || !current_next_indicator) {
-#ifdef TS_PMT_LOG
-      printf ("ts_demux: section_syntax_indicator != 1 "
-              "|| !current_next_indicator\n");
-#endif
+    /* allocate space for largest possible section */
+    if (!this->pmt[program_count])
+      this->pmt[program_count] = malloc (4098);
+    if (!this->pmt[program_count])
+      return;
+    this->pmt_write_pos[program_count] = 0;
+    /* offset the section by n + 1 bytes. this is sometimes used to let it end
+       at an exact TS packet boundary */
+    plen -= (unsigned int)pkt[0] + 1;
+    pkt += (unsigned int)pkt[0] + 1;
+    if (plen < 1) {
+      xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
+        "demux_ts: demux error! PMT with invalid pointer\n");
       return;
     }
+  } else {
+    if (!this->pmt_write_pos[program_count])
+      return;
+  }
+  if (plen > 4098 - this->pmt_write_pos[program_count])
+    plen = 4098 - this->pmt_write_pos[program_count];
+  memcpy (this->pmt[program_count] + this->pmt_write_pos[program_count], pkt, plen);
+  this->pmt_write_pos[program_count] += plen;
 
-    if (program_number != this->program_number[program_count]) {
-      /* several programs can share the same PMT pid */
+  /* lets see if we got the section length already */
+  pkt = this->pmt[program_count];
+  if (this->pmt_write_pos[program_count] < 3)
+    return;
+  section_length = ((((uint32_t)pkt[1] << 8) | pkt[2]) & 0xfff) + 3;
+  /* this should be at least the head plus crc */
+  if (section_length < 8 + 4) {
+    this->pmt_write_pos[program_count] = 0;
+    return;
+  }
+
+  /* lets see if we got the section complete */
+  if (this->pmt_write_pos[program_count] < section_length)
+    return;
+
+  /* OK now parse it */
+  this->pmt_write_pos[program_count] = 0;
+#ifdef TS_PMT_LOG
+  table_id                  = (unsigned int)pkt[0];
+#endif
+  section_syntax_indicator  = (pkt[1] >> 7) & 0x01;
+  program_number            = ((uint32_t)pkt[3] << 8) | pkt[4];
+#ifdef TS_PMT_LOG
+  version_number            = ((uint32_t)pkt[5] >> 1) & 0x1f;
+#endif
+  current_next_indicator    =  pkt[5] & 0x01;
+  section_number            =  pkt[6];
+  last_section_number       =  pkt[7];
+
+#ifdef TS_PMT_LOG
+  printf ("demux_ts: PMT table_id: %.2x\n", table_id);
+  printf ("              section_syntax: %d\n", section_syntax_indicator);
+  printf ("              section_length: %d (%#.3x)\n", section_length, section_length);
+  printf ("              program_number: %#.4x\n", program_number);
+  printf ("              version_number: %d\n", version_number);
+  printf ("              c/n indicator: %d\n", current_next_indicator);
+  printf ("              section_number: %d\n", section_number);
+  printf ("              last_section_number: %d\n", last_section_number);
+#endif
+
+  if ((section_syntax_indicator != 1) || !current_next_indicator)
+    return;
+
+  if ((section_number != 0) || (last_section_number != 0)) {
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
+      "demux_ts: FIXME (unsupported) PMT consists of multiple (%d) sections\n", last_section_number);
+    return;
+  }
+
+  /* Check CRC. */
+  crc32  = (uint32_t) pkt[section_length - 4] << 24;
+  crc32 |= (uint32_t) pkt[section_length - 3] << 16;
+  crc32 |= (uint32_t) pkt[section_length - 2] << 8;
+  crc32 |= (uint32_t) pkt[section_length - 1];
+
+  calc_crc32 = htonl (av_crc (this->class->av_crc, 0xffffffff, pkt, section_length - 4));
+  if (crc32 != calc_crc32) {
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
+      "demux_ts: demux error! PMT with invalid CRC32: packet_crc32: %#.8x calc_crc32: %#.8x\n",
+      crc32, calc_crc32);
+    return;
+  }
+#ifdef TS_PMT_LOG
+  printf ("demux_ts: PMT CRC32 ok.\n");
+#endif
+
+  if (program_number != this->program_number[program_count]) {
+    /* several programs can share the same PMT pid */
 #ifdef TS_PMT_LOG
 printf("Program Number is %i, looking for %i\n",program_number,this->program_number[program_count]);
       printf ("ts_demux: waiting for next PMT on this PID...\n");
-#endif
-      return;
-    }
-
-    if ((section_number != 0) || (last_section_number != 0)) {
-      xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
-	       "demux_ts: FIXME (unsupported) PMT consists of multiple (%d) sections\n", last_section_number);
-      return;
-    }
-
-  }
-
-  if (!this->pmt[program_count]) {
-    /* not the first TS packet of a PMT, or the calloc didn't work */
-#ifdef TS_PMT_LOG
-    printf ("ts_demux: not the first TS packet of a PMT...\n");
 #endif
     return;
   }
 
-  if (!pusi){
-    section_length = (this->pmt[program_count][1] << 8
-		      | this->pmt[program_count][2]) & 0x03ff;
-  }
-
-  count=ts_payloadsize(originalPkt);
-
-  ptr = originalPkt+offset+(PKT_SIZE-count);
-  len = count-offset;
-  memcpy (this->pmt_write_ptr[program_count], ptr, len);
-  this->pmt_write_ptr[program_count] +=len;
-
+  if (crc32 == this->last_pmt_crc) {
 #ifdef TS_PMT_LOG
-  printf ("ts_demux: wr_ptr: %p, will be %p when finished\n",
-	  this->pmt_write_ptr[program_count],
-	  this->pmt[program_count] + section_length);
-#endif
-  if (this->pmt_write_ptr[program_count] < this->pmt[program_count]
-      + section_length) {
-    /* didn't get all TS packets for this section yet */
-#ifdef TS_PMT_LOG
-    printf ("ts_demux: didn't get all PMT TS packets yet...\n");
+    printf("demux_ts: PMT with CRC32=%d already parsed. Skipping.\n", crc32);
 #endif
     return;
   }
+  this->last_pmt_crc = crc32;
 
-  if (!section_length) {
-    free (this->pmt[program_count]);
-    this->pmt[program_count] = NULL;
-#ifdef TS_PMT_LOG
-    printf ("ts_demux: eek, zero-length section?\n");
-#endif
-    return;
-  }
-
-#ifdef TS_PMT_LOG
-  printf ("ts_demux: have all TS packets for the PMT section\n");
-#endif
-
-  crc32  = (uint32_t) this->pmt[program_count][section_length+3-4] << 24;
-  crc32 |= (uint32_t) this->pmt[program_count][section_length+3-3] << 16;
-  crc32 |= (uint32_t) this->pmt[program_count][section_length+3-2] << 8;
-  crc32 |= (uint32_t) this->pmt[program_count][section_length+3-1] ;
-
-  /* Check CRC. */
-  calc_crc32 = htonl(av_crc(this->class->av_crc, 0xffffffff,
-			    this->pmt[program_count], section_length+3-4));
-
-  if (crc32 != calc_crc32) {
-    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
-	     "demux_ts: demux error! PMT with invalid CRC32: packet_crc32: %#.8x calc_crc32: %#.8x\n",
-	     crc32,calc_crc32);
-    return;
-  }
-  else {
-#ifdef TS_PMT_LOG
-    printf ("demux_ts: PMT CRC32 ok.\n");
-#endif
-    if ( crc32==this->last_pmt_crc ) {
-#ifdef TS_PMT_LOG
-      printf("demux_ts: PMT with CRC32=%d already parsed. Skipping.\n", crc32);
-#endif
-      return;
-    }
-    else {
-#ifdef TS_PMT_LOG
-      printf("demux_ts: new PMT, parsing...\n");
-#endif
-      this->last_pmt_crc = crc32;
-    }
-  }
+  /* dont "parse" the CRC */
+  section_length -= 4;
 
   /*
    * Forget the current video, audio and subtitle PIDs; if the PMT has not
@@ -1587,8 +1627,7 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
    * ES definitions start here...we are going to learn upto one video
    * PID and one audio PID.
    */
-  program_info_length = ((this->pmt[program_count][10] << 8)
-                       | this->pmt[program_count][11]) & 0x0fff;
+  program_info_length = ((pkt[10] << 8) | pkt[11]) & 0x0fff;
 
 /* Program info descriptor is currently just ignored.
  * printf ("demux_ts: program_info_desc: ");
@@ -1596,8 +1635,8 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
  *       printf ("%.2x ", this->pmt[program_count][12+i]);
  * printf ("\n");
  */
-  stream = &this->pmt[program_count][12] + program_info_length;
-  coded_length = 13 + program_info_length;
+  stream = pkt + 12 + program_info_length;
+  coded_length = 12 + program_info_length;
   if (coded_length > section_length) {
     xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
 	     "demux error! PMT with inconsistent progInfo length\n");
@@ -1817,6 +1856,11 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
           }
         }
       }
+
+      xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+              "demux_ts: PMT unknown stream_type: 0x%.2x pid: 0x%.4x\n",
+              stream[0], pid);
+
 #ifdef TS_PMT_LOG
       printf ("demux_ts: PMT unknown stream_type: 0x%.2x pid: 0x%.4x\n",
               stream[0], pid);
@@ -2227,8 +2271,8 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
 
   /* PAT */
   if (pid == 0) {
-    demux_ts_parse_pat(this, originalPkt, originalPkt+data_offset-4,
-		       payload_unit_start_indicator);
+    demux_ts_parse_pat(this, originalPkt, originalPkt + data_offset,
+		       payload_unit_start_indicator, PKT_SIZE - data_offset);
     return;
   }
 
@@ -2243,8 +2287,8 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
               this->program_number[program_count],
               this->pmt_pid[program_count]);
 #endif
-      demux_ts_parse_pmt (this, originalPkt, originalPkt+data_offset-4,
-                          payload_unit_start_indicator,
+      demux_ts_parse_pmt (this, originalPkt, originalPkt + data_offset,
+                          payload_unit_start_indicator, PKT_SIZE - data_offset,
                           program_count);
       return;
     }
@@ -2643,7 +2687,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
     this->program_number[i]          = INVALID_PROGRAM;
     this->pmt_pid[i]                 = INVALID_PID;
     this->pmt[i]                     = NULL;
-    this->pmt_write_ptr[i]           = NULL;
+    this->pmt_write_pos[i]           = 0;
   }
 
   this->scrambled_npids = 0;
