@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2005 the xine project
+ * Copyright (C) 2000-2011 the xine project
  * March 2003 - Miguel Freitas
  * This plugin was sponsored by 1Control
  *
@@ -114,10 +114,10 @@
 #define LOG
 */
 
-#include "xine_internal.h"
-#include "xineutils.h"
-#include "compat.h"
-#include "input_plugin.h"
+#include <xine/xine_internal.h>
+#include <xine/xineutils.h>
+#include <xine/compat.h>
+#include <xine/input_plugin.h>
 
 #define PVR_DEVICE        "/dev/video0"
 
@@ -427,8 +427,9 @@ static uint32_t pvr_plugin_get_capabilities (input_plugin_t *this_gen) {
 }
 
 
-static off_t pvr_plugin_read (input_plugin_t *this_gen, char *buf, off_t len) {
+static off_t pvr_plugin_read (input_plugin_t *this_gen, void *buf_gen, off_t len) {
   /*pvr_input_plugin_t *this = (pvr_input_plugin_t *) this_gen;*/
+  char *buf = (char *)buf_gen;
 
   if (len < 4)
     return -1;
@@ -1537,22 +1538,6 @@ static input_plugin_t *pvr_class_get_instance (input_class_t *cls_gen, xine_stre
 /*
  * plugin class functions
  */
-
-static const char *pvr_class_get_description (input_class_t *this_gen) {
-  return _("WinTV-PVR 250/350 input plugin");
-}
-
-static const char *pvr_class_get_identifier (input_class_t *this_gen) {
-  return "pvr";
-}
-
-
-static void pvr_class_dispose (input_class_t *this_gen) {
-  pvr_input_class_t  *this = (pvr_input_class_t *) this_gen;
-
-  free (this);
-}
-
 static void *init_plugin (xine_t *xine, void *data) {
 
   pvr_input_class_t  *this;
@@ -1571,11 +1556,11 @@ static void *init_plugin (xine_t *xine, void *data) {
 				    NULL);
 
   this->input_class.get_instance       = pvr_class_get_instance;
-  this->input_class.get_identifier     = pvr_class_get_identifier;
-  this->input_class.get_description    = pvr_class_get_description;
+  this->input_class.identifier         = "pvr";
+  this->input_class.description        = N_("WinTV-PVR 250/350 input plugin");
   this->input_class.get_dir            = NULL;
   this->input_class.get_autoplay_list  = NULL;
-  this->input_class.dispose            = pvr_class_dispose;
+  this->input_class.dispose            = default_input_class_dispose;
   this->input_class.eject_media        = NULL;
 
   return this;
@@ -1587,7 +1572,7 @@ static void *init_plugin (xine_t *xine, void *data) {
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, "pvr", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 18, "pvr", XINE_VERSION_CODE, NULL, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
