@@ -1,5 +1,5 @@
 /* kate: space-indent on; indent-width 2; mixedindent off; indent-mode cstyle; remove-trailing-space on;
- * Copyright (C) 2008 the xine project
+ * Copyright (C) 2008-2013 the xine project
  * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
  *
  * This file is part of xine, a free video player.
@@ -383,7 +383,6 @@ reset_sequence (sequence_t * sequence)
   dpb_reset (sequence);
   memset (&sequence->cur_pic, 0, sizeof (dpb_frame_t));
   sequence->reset = VO_NEW_SEQUENCE_FLAG;
-  sequence->color_standard = VDP_COLOR_STANDARD_ITUR_BT_601;
 }
 
 
@@ -601,6 +600,7 @@ static void
 vui_parameters (sequence_t * seq, vui_param_t * vui)
 {
   bits_reader_t *br = &seq->br;
+  int xine_color_matrix = 4; /* undefined, mpeg range */
 
   vui->aspect_ratio_info = read_bits (br, 1);
   lprintf ("aspect_ratio_info_present_flag = %d\n", vui->aspect_ratio_info);
@@ -621,27 +621,17 @@ vui_parameters (sequence_t * seq, vui_param_t * vui)
   if (read_bits (br, 1))
   {				/* video_signal_type_present_flag */
     skip_bits (br, 3);		/*video_format */
-    skip_bits (br, 1);		/*video_full_range_flag */
+    xine_color_matrix |= read_bits (br, 1);  /*video_full_range_flag */
     vui->colour_desc = read_bits (br, 1);
     lprintf ("colour_desc = %d\n", vui->colour_desc);
     if (vui->colour_desc)
     {
-      vui->colour_primaries = read_bits (br, 8);
-      lprintf ("colour_primaries = %d\n", vui->colour_primaries);
+      skip_bits (br, 8);        /* colour_primaries */
       skip_bits (br, 8);	/* transfer_characteristics */
-      skip_bits (br, 8);	/* matrix_coefficients */
-      switch (vui->colour_primaries)
-      {
-      case 1:
-	seq->color_standard = VDP_COLOR_STANDARD_ITUR_BT_709;
-	break;
-      case 6:
-      case 7:
-	seq->color_standard = VDP_COLOR_STANDARD_SMPTE_240M;
-	break;
-      }
+      xine_color_matrix = (xine_color_matrix & 1) | (read_bits (br, 8) << 1);  /* matrix_coefficients */
     }
   }
+  VO_SET_FLAGS_CM (xine_color_matrix, seq->color_matrix);
   if (read_bits (br, 1))
   {				/* chroma_loc_info_present_flag */
     read_exp_ue (br);		/* chroma_sample_loc_type_top_field */
@@ -825,6 +815,8 @@ seq_parameter_set_data (vdpau_h264_alter_decoder_t * this_gen)
     lprintf ("mb_adaptive_frame_field_flag = %u\n",
 	     sp->mb_adaptive_frame_field_flag);
   }
+  else
+    sp->mb_adaptive_frame_field_flag = 0;
   sp->direct_8x8_inference_flag = read_bits (&seq->br, 1);
   lprintf ("direct_8x8_inference_flag = %u\n", sp->direct_8x8_inference_flag);
   sp->frame_cropping_flag = read_bits (&seq->br, 1);
@@ -1222,6 +1214,7 @@ slice_header (vdpau_h264_alter_decoder_t * this_gen, uint8_t nal_ref_idc,
       && !seq->cur_pic.completed)
     seq->startup_frame = 1;
 
+  lprintf ("seq_parameter_set_id = %u\n", pic->seq_parameter_set_id);
   sp = seq->seq_param[pic->seq_parameter_set_id];
   if (sp->separate_colour_plane_flag)
     read_bits (&seq->br, 2);	/* colour_plane_id */
@@ -1666,7 +1659,7 @@ decode_render (vdpau_h264_alter_decoder_t * vd, int bad_frame)
 					seq->coded_width, seq->coded_height,
 					seq->ratio, XINE_IMGFMT_VDPAU,
 					VO_BOTH_FIELDS | seq->chroma | seq->
-					reset);
+					reset | seq->color_matrix);
     seq->reset = 0;
     img->drawn = 0;
   }
@@ -1849,8 +1842,6 @@ decode_render (vdpau_h264_alter_decoder_t * vd, int bad_frame)
     xine_event_send (vd->stream, &event);
   }
 
-  accel->color_standard = seq->color_standard;
-
   if (seq->cur_pic.completed == PICTURE_DONE)
   {
     seq->cur_pic.pts = seq->pic_pts;
