@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2009 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -59,10 +59,10 @@
 */
 
 #include "xine.h"
-#include "xine_internal.h"
-#include "video_out.h"
-#include "xineutils.h"
-#include "vo_scale.h"
+#include <xine/xine_internal.h>
+#include <xine/video_out.h>
+#include <xine/xineutils.h>
+#include <xine/vo_scale.h>
 
 #ifdef HAVE_X11
 #include <X11/Xlib.h>
@@ -384,7 +384,7 @@ static int sdl_set_property (vo_driver_t *this_gen,
       value = XINE_VO_ASPECT_AUTO;
     this->sc.user_ratio = value;
     xprintf(this->xine, XINE_VERBOSITY_DEBUG,
-	    "video_out_sdl: aspect ratio changed to %s\n", _x_vo_scale_aspect_ratio_name(value));
+	    "video_out_sdl: aspect ratio changed to %s\n", _x_vo_scale_aspect_ratio_name_table[value]);
 
     sdl_compute_ideal_size (this);
     this->sc.force_redraw = 1;
@@ -577,19 +577,6 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *vi
 /**
  * Class Functions
  */
-static char* get_identifier (video_driver_class_t *this_gen) {
-  return "SDL";
-}
-
-static char* get_description (video_driver_class_t *this_gen) {
-  return _("xine video output plugin using the Simple Direct Media Layer");
-}
-
-static void dispose_class (video_driver_class_t *this_gen) {
-  free(this_gen);
-}
-
-
 static void *init_class (xine_t *xine, void *visual_gen) {
   /* x11_visual_t     *visual = (x11_visual_t *) visual_gen; */
   sdl_class_t      *this;
@@ -605,9 +592,9 @@ static void *init_class (xine_t *xine, void *visual_gen) {
   this = (sdl_class_t*) calloc(1, sizeof(sdl_class_t));
 
   this->driver_class.open_plugin      = open_plugin;
-  this->driver_class.get_identifier   = get_identifier;
-  this->driver_class.get_description  = get_description;
-  this->driver_class.dispose          = dispose_class;
+  this->driver_class.identifier       = "SDL";
+  this->driver_class.description      = N_("xine video output plugin using the Simple Direct Media Layer");
+  this->driver_class.dispose          = default_video_driver_class_dispose;
 
   this->config                        = xine->config;
   this->xine                          = xine;
@@ -622,6 +609,6 @@ static const vo_info_t vo_info_sdl = {
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_VIDEO_OUT, 21, "sdl", XINE_VERSION_CODE, &vo_info_sdl, init_class },
+  { PLUGIN_VIDEO_OUT, 22, "sdl", XINE_VERSION_CODE, &vo_info_sdl, init_class },
   { PLUGIN_NONE, 0, "" , 0 , NULL, NULL}
 };
