@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004, 2008 the xine project
+ * Copyright (C) 2000-2012 the xine project
  * Copyright (C) 2004 the Unichrome project
  *
  * This file is part of xine, a free video player.
@@ -1794,6 +1794,8 @@ static void xxmc_display_frame (vo_driver_t *this_gen, vo_frame_t *frame_gen)
 static int xxmc_get_property (vo_driver_t *this_gen, int property) {
   xxmc_driver_t *this = (xxmc_driver_t *) this_gen;
 
+  if ((property < 0) || (property >= VO_NUM_PROPERTIES)) return 0;
+
   switch (property) {
   case VO_PROP_WINDOW_WIDTH:
     this->props[property].value = this->sc.gui_width;
@@ -1844,6 +1846,8 @@ static int xxmc_set_property (vo_driver_t *this_gen,
 			      int property, int value) {
   xxmc_driver_t *this = (xxmc_driver_t *) this_gen;
 
+  if ((property < 0) || (property >= VO_NUM_PROPERTIES)) return 0;
+
   if (this->props[property].atom != None) {
 
     /* value is out of bound */
@@ -1932,6 +1936,10 @@ static void xxmc_get_property_min_max (vo_driver_t *this_gen,
 				       int property, int *min, int *max) {
   xxmc_driver_t *this = (xxmc_driver_t *) this_gen;
 
+  if ((property < 0) || (property >= VO_NUM_PROPERTIES)) {
+    *min = *max = 0;
+    return;
+  }
   *min = this->props[property].min;
   *max = this->props[property].max;
 }
