@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003, 2007 the xine project
+ * Copyright (C) 2003-2009 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -49,7 +49,7 @@
 #define LOG
 */
 
-#include "xine_internal.h"
+#include <xine/xine_internal.h>
 #include "xcbosd.h"
 
 struct xcbosd
